@@ -173,8 +173,28 @@ static void logger_file(const char *line, void *user)
 static void logger_stdout_sync(const char *line, void *user)
 {
 	(void)user;
-	puts(line);
-	fflush(stdout);
+
+	size_t length = strlen(line);
+	wchar_t *wide = malloc(length * sizeof (*wide));
+	mem_zero(wide, length * sizeof *wide);
+
+	const char *p = line;
+	int wlen = 0;
+	for(int codepoint = 0; (codepoint = str_utf8_decode(&p)); wlen++)
+	{
+		if(codepoint < 0)
+			return;
+
+		char u16[4] = {0};
+		if(str_utf16le_encode(u16, codepoint) != 2)
+			return;
+
+		mem_copy(&wide[wlen], u16, 2);
+	}
+
+	HANDLE console = GetStdHandle(STD_OUTPUT_HANDLE);
+	WriteConsoleW(console, wide, wlen, NULL, NULL);
+	WriteConsoleA(console, "\n", 1, NULL, NULL);
 }
 #endif
 
@@ -732,12 +752,12 @@ void thread_yield()
 #endif
 }
 
-void thread_sleep(int milliseconds)
+void thread_sleep(int microseconds)
 {
 #if defined(CONF_FAMILY_UNIX)
-	usleep(milliseconds*1000);
+	usleep(microseconds);
 #elif defined(CONF_FAMILY_WINDOWS)
-	Sleep(milliseconds);
+	Sleep(microseconds/1000);
 #else
 	#error not implemented
 #endif
@@ -1326,7 +1346,6 @@ NETSOCKET net_udp_create(NETADDR bindaddr)
 	NETSOCKET sock = invalid_socket;
 	NETADDR tmpbindaddr = bindaddr;
 	int broadcast = 1;
-	int recvsize = 65536;
 
 	if(bindaddr.type&NETTYPE_IPV4)
 	{
@@ -1345,9 +1364,6 @@ NETSOCKET net_udp_create(NETADDR bindaddr)
 			/* set broadcast */
 			setsockopt(socket, SOL_SOCKET, SO_BROADCAST, (const char*)&broadcast, sizeof(broadcast));
 
-			/* set receive buffer size */
-			setsockopt(socket, SOL_SOCKET, SO_RCVBUF, (char*)&recvsize, sizeof(recvsize));
-
 			{
 				/* set DSCP/TOS */
 				int iptos = 0x10 /* IPTOS_LOWDELAY */;
@@ -1393,9 +1409,6 @@ NETSOCKET net_udp_create(NETADDR bindaddr)
 			/* set broadcast */
 			setsockopt(socket, SOL_SOCKET, SO_BROADCAST, (const char*)&broadcast, sizeof(broadcast));
 
-			/* set receive buffer size */
-			setsockopt(socket, SOL_SOCKET, SO_RCVBUF, (char*)&recvsize, sizeof(recvsize));
-
 			{
 				/* set DSCP/TOS */
 				int iptos = 0x10 /* IPTOS_LOWDELAY */;
@@ -1502,30 +1515,85 @@ int net_udp_send(NETSOCKET sock, const NETADDR *addr, const void *data, int size
 #endif /* FUZZING */
 }
 
-int net_udp_recv(NETSOCKET sock, NETADDR *addr, void *data, int maxsize)
+void net_init_mmsgs(MMSGS* m)
+{
+#if defined(CONF_PLATFORM_LINUX)
+	int i;
+	m->pos = 0;
+	m->size = 0;
+	mem_zero(m->msgs, sizeof(m->msgs));
+	mem_zero(m->iovecs, sizeof(m->iovecs));
+	mem_zero(m->sockaddrs, sizeof(m->sockaddrs));
+	for(i = 0; i < VLEN; ++i)
+	{
+		m->iovecs[i].iov_base = m->bufs[i];
+		m->iovecs[i].iov_len = PACKETSIZE;
+		m->msgs[i].msg_hdr.msg_iov = &(m->iovecs[i]);
+		m->msgs[i].msg_hdr.msg_iovlen = 1;
+		m->msgs[i].msg_hdr.msg_name = &(m->sockaddrs[i]);
+		m->msgs[i].msg_hdr.msg_namelen = sizeof(m->sockaddrs[i]);
+	}
+#endif
+}
+
+int net_udp_recv(NETSOCKET sock, NETADDR *addr, void *buffer, int maxsize, MMSGS* m, unsigned char **data)
 {
 #ifndef FUZZING
 	char sockaddrbuf[128];
-	socklen_t fromlen;// = sizeof(sockaddrbuf);
 	int bytes = 0;
 
+#if defined(CONF_PLATFORM_LINUX)
+	if(sock.ipv4sock >= 0)
+	{
+		if(m->pos >= m->size)
+		{
+			m->size = recvmmsg(sock.ipv4sock, m->msgs, VLEN, 0, NULL);
+			m->pos = 0;
+		}
+	}
+
+	if(sock.ipv6sock >= 0)
+	{
+		if(m->pos >= m->size)
+		{
+			m->size = recvmmsg(sock.ipv6sock, m->msgs, VLEN, 0, NULL);
+			m->pos = 0;
+		}
+	}
+
+	if(m->pos < m->size)
+	{
+		sockaddr_to_netaddr((struct sockaddr *)&(m->sockaddrs[m->pos]), addr);
+		// TODO: network_stats
+		//network_stats.recv_bytes += bytes;
+		//network_stats.recv_packets++;
+		bytes = m->msgs[m->pos].msg_len;
+		*data = (unsigned char*)m->bufs[m->pos];
+		m->pos++;
+		return bytes;
+	}
+#else
 	if(bytes == 0 && sock.ipv4sock >= 0)
 	{
-		fromlen = sizeof(struct sockaddr_in);
-		bytes = recvfrom(sock.ipv4sock, (char*)data, maxsize, 0, (struct sockaddr *)&sockaddrbuf, &fromlen);
+		socklen_t fromlen = sizeof(struct sockaddr_in);
+		bytes = recvfrom(sock.ipv4sock, (char*)buffer, maxsize, 0, (struct sockaddr *)&sockaddrbuf, &fromlen);
+		*data = buffer;
 	}
 
 	if(bytes <= 0 && sock.ipv6sock >= 0)
 	{
-		fromlen = sizeof(struct sockaddr_in6);
-		bytes = recvfrom(sock.ipv6sock, (char*)data, maxsize, 0, (struct sockaddr *)&sockaddrbuf, &fromlen);
+		socklen_t fromlen = sizeof(struct sockaddr_in6);
+		bytes = recvfrom(sock.ipv6sock, (char*)buffer, maxsize, 0, (struct sockaddr *)&sockaddrbuf, &fromlen);
+		*data = buffer;
 	}
+#endif
 
 #if defined(CONF_WEBSOCKETS)
 	if(bytes <= 0 && sock.web_ipv4sock >= 0)
 	{
-		fromlen = sizeof(struct sockaddr);
-		bytes = websocket_recv(sock.web_ipv4sock, data, maxsize, (struct sockaddr_in *)&sockaddrbuf, fromlen);
+		socklen_t fromlen = sizeof(struct sockaddr);
+		bytes = websocket_recv(sock.web_ipv4sock, buffer, maxsize, (struct sockaddr_in *)&sockaddrbuf, fromlen);
+		*data = buffer;
 		((struct sockaddr_in *)&sockaddrbuf)->sin_family = AF_WEBSOCKET_INET;
 	}
 #endif
@@ -1540,7 +1608,7 @@ int net_udp_recv(NETSOCKET sock, NETADDR *addr, void *data, int maxsize)
 	else if(bytes == 0)
 		return 0;
 	return -1; /* error */
-#else
+#else /* ifdef FUZZING */
 	addr->type = NETTYPE_IPV4;
 	addr->port = 11111;
 	addr->ip[0] = 127;
@@ -1557,7 +1625,8 @@ int net_udp_recv(NETSOCKET sock, NETADDR *addr, void *data, int maxsize)
 			break;
 		}
 
-		((unsigned char*)data)[CurrentData] = gs_NetData[gs_NetPosition];
+		((unsigned char*)buffer)[CurrentData] = gs_NetData[gs_NetPosition];
+		*data = buffer;
 		CurrentData++;
 		gs_NetPosition++;
 	}
@@ -2298,7 +2367,7 @@ int str_comp_nocase(const char *a, const char *b)
 #endif
 }
 
-int str_comp_nocase_num(const char *a, const char *b, const int num)
+int str_comp_nocase_num(const char *a, const char *b, int num)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	return _strnicmp(a, b, num);
@@ -2312,7 +2381,7 @@ int str_comp(const char *a, const char *b)
 	return strcmp(a, b);
 }
 
-int str_comp_num(const char *a, const char *b, const int num)
+int str_comp_num(const char *a, const char *b, int num)
 {
 	return strncmp(a, b, num);
 }
@@ -2652,6 +2721,63 @@ int str_toint(const char *str) { return atoi(str); }
 int str_toint_base(const char *str, int base) { return strtol(str, NULL, base); }
 float str_tofloat(const char *str) { return atof(str); }
 
+int str_utf8_comp_nocase(const char *a, const char *b)
+{
+	int code_a;
+	int code_b;
+
+	while(*a && *b)
+	{
+		code_a = str_utf8_tolower(str_utf8_decode(&a));
+		code_b = str_utf8_tolower(str_utf8_decode(&b));
+
+		if(code_a != code_b)
+			return code_a - code_b;
+	}
+	return (unsigned char)*a - (unsigned char)*b;
+}
+
+int str_utf8_comp_nocase_num(const char *a, const char *b, int num)
+{
+	int code_a;
+	int code_b;
+	const char *old_a = a;
+
+	while(*a && *b)
+	{
+		if(a - old_a >= num)
+			return 0;
+
+		code_a = str_utf8_tolower(str_utf8_decode(&a));
+		code_b = str_utf8_tolower(str_utf8_decode(&b));
+
+		if(code_a != code_b)
+			return code_a - code_b;
+	}
+
+	return (unsigned char)*a - (unsigned char)*b;
+}
+
+const char *str_utf8_find_nocase(const char *haystack, const char *needle)
+{
+	while(*haystack) /* native implementation */
+	{
+		const char *a = haystack;
+		const char *b = needle;
+		const char *a_next = a;
+		const char *b_next = b;
+		while(*a && *b && str_utf8_tolower(str_utf8_decode(&a_next)) == str_utf8_tolower(str_utf8_decode(&b_next)))
+		{
+			a = a_next;
+			b = b_next;
+		}
+		if(!(*b))
+			return haystack;
+		str_utf8_decode(&haystack);
+	}
+
+	return 0;
+}
 
 int str_utf8_isspace(int code)
 {
@@ -2789,6 +2915,32 @@ int str_utf8_encode(char *ptr, int chr)
 	return 0;
 }
 
+int str_utf16le_encode(char *ptr, int chr)
+{
+	if(chr < 0x10000)
+	{
+		ptr[0] = chr;
+		ptr[1] = chr >> 0x8;
+		return 2;
+	}
+	else if(chr <= 0x10FFFF)
+	{
+		int U = chr - 0x10000;
+		int W1 = 0xD800, W2 = 0xDC00;
+
+		W1 |= ((U >> 10) & 0x3FF);
+		W2 |= (U & 0x3FF);
+
+		ptr[0] = W1;
+		ptr[1] = W1 >> 0x8;
+		ptr[2] = W2;
+		ptr[3] = W2 >> 0x8;
+		return 4;
+	}
+
+	return 0;
+}
+
 static unsigned char str_byte_next(const char **ptr)
 {
 	unsigned char byte = **ptr;
