@@ -55,6 +55,35 @@ TEST(Str, Utf8CompConfusables)
 	EXPECT_TRUE(str_utf8_comp_confusable("aceiou", "ąçęįǫų") == 0);
 }
 
+TEST(Str, Utf8ToLower)
+{
+	EXPECT_TRUE(str_utf8_tolower('A') == 'a');
+	EXPECT_TRUE(str_utf8_tolower('z') == 'z');
+	EXPECT_TRUE(str_utf8_tolower(192) == 224); // À -> à
+	EXPECT_TRUE(str_utf8_tolower(7882) == 7883); // Ị -> ị
+
+	EXPECT_TRUE(str_utf8_comp_nocase("ÖlÜ", "ölü") == 0);
+	EXPECT_TRUE(str_utf8_comp_nocase("ÜlÖ", "ölü") > 0); // ü > ö
+	EXPECT_TRUE(str_utf8_comp_nocase("ÖlÜ", "ölüa") < 0); // NULL < a
+	EXPECT_TRUE(str_utf8_comp_nocase("ölüa", "ÖlÜ") > 0); // a < NULL
+
+	const char a[2] = {-128, 0};
+	const char b[2] = {0, 0};
+	EXPECT_TRUE(str_utf8_comp_nocase(a, b) > 0);
+	EXPECT_TRUE(str_utf8_comp_nocase(b, a) < 0);
+
+	EXPECT_TRUE(str_utf8_comp_nocase_num("ÖlÜ", "ölüa", 3) == 0);
+	EXPECT_TRUE(str_utf8_comp_nocase_num("ÖlÜ", "ölüa", 4) != 0);
+	EXPECT_TRUE(str_utf8_comp_nocase_num("a", "z", 0) == 0);
+	EXPECT_TRUE(str_utf8_comp_nocase_num("a", "z", 1) != 0);
+
+	const char str[] = "ÄÖÜ";
+	EXPECT_TRUE(str_utf8_find_nocase(str, "ä") == str);
+	EXPECT_TRUE(str_utf8_find_nocase(str, "ö") == str+2);
+	EXPECT_TRUE(str_utf8_find_nocase(str, "ü") == str+4);
+	EXPECT_TRUE(str_utf8_find_nocase(str, "z") == NULL);
+}
+
 TEST(Str, Startswith)
 {
 	EXPECT_TRUE(str_startswith("abcdef", "abc"));
